from enry.definitions import get_color, get_language_by_shebang, get_language, get_language_by_content, \
    get_language_by_emacs_modeline, get_language_by_extension, get_language_by_filename, get_language_by_modeline, \
    get_language_by_vim_modeline, get_languages, get_mime_type, is_binary, is_configuration, is_documentation, \
    is_dot_file, \
    is_generated, is_image, is_vendor

__all__ = [
    "get_color",
    "get_language",
    "get_languages",
    "get_mime_type",
    "get_language_by_vim_modeline",
    "get_language_by_extension",
    "get_language_by_content",
    "get_language_by_emacs_modeline",
    "get_language_by_modeline",
    "get_language_by_filename",
    "get_language_by_shebang",
    "is_vendor",
    "is_binary",
    "is_image",
    "is_generated",
    "is_documentation",
    "is_dot_file",
    "is_configuration",
]
